@@ -1,6 +1,6 @@
 <?php declare(strict_types = 0);
 /*
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -47,7 +47,7 @@ if ($host_is_discovered) {
 				(new CUrl('host_prototypes.php'))
 					->setArgument('form', 'update')
 					->setArgument('parent_discoveryid', $data['host']['discoveryRule']['itemid'])
-					->setArgument('hostid', $data['host']['hostDiscovery']['parent_hostid'])
+					->setArgument('hostid', $data['host']['discoveryData']['parent_hostid'])
 					->setArgument('context', 'host')
 			))->setAttribute('target', '_blank');
 		}
@@ -108,7 +108,9 @@ if ($data['host']['parentTemplates']) {
 		->setHeader([_('Name'), _('Actions')])
 		->setId('linked-templates')
 		->addClass(ZBX_STYLE_TABLE_FORMS)
-		->addStyle('width: '.ZBX_TEXTAREA_STANDARD_WIDTH.'px;');
+		->addStyle('width: '.ZBX_TEXTAREA_STANDARD_WIDTH.'px;')
+		->setAttribute('data-field-type', 'array')
+		->setAttribute('data-field-name', 'templates');
 
 	foreach ($data['host']['parentTemplates'] as $template) {
 		if ($data['user']['can_edit_templates']
@@ -119,10 +121,7 @@ if ($data['host']['parentTemplates']) {
 				->setArgument('templateid', $template['templateid'])
 				->getUrl();
 
-			$template_link = (new CLink($template['name'], $template_url))
-				->addClass('js-edit-template')
-				->setAttribute('data-templateid', $template['templateid'])
-				->setAttribute('data-action', 'template.edit');
+			$template_link = new CLink($template['name'], $template_url);
 		}
 		else {
 			$template_link = new CSpan($template['name']);
@@ -178,8 +177,8 @@ $templates_field_items[] = (new CMultiSelect([
 ]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH);
 
 $disabled_by_lld_icon = $data['host']['status'] == HOST_STATUS_NOT_MONITORED
-		&& array_key_exists('hostDiscovery', $data['host']) && $data['host']['hostDiscovery']
-		&& $data['host']['hostDiscovery']['disable_source'] == ZBX_DISABLE_SOURCE_LLD
+		&& array_key_exists('discoveryData', $data['host']) && $data['host']['discoveryData']
+		&& $data['host']['discoveryData']['disable_source'] == ZBX_DISABLE_SOURCE_LLD
 	? makeWarningIcon(_('Disabled automatically by an LLD rule.'))
 	: null;
 
@@ -207,9 +206,11 @@ $host_tab
 		new CFormField(
 			(new CMultiSelect([
 				'name' => 'groups[]',
+				'new_item_name' => 'groups_new[]',
 				'object_name' => 'hostGroup',
 				'readonly' => $host_is_discovered,
 				'add_new' => (CWebUser::$data['type'] == USER_TYPE_SUPER_ADMIN),
+				'maxlength' => DB::getFieldLength('hstgrp', 'name'),
 				'data' => $data['groups_ms'],
 				'popup' => [
 					'parameters' => [
@@ -230,25 +231,41 @@ $host_tab
 		new CLabel(_('Interfaces')),
 		new CFormField([
 			(new CDiv([
-				renderInterfaceHeaders(),
-				$agent_interfaces,
-				$snmp_interfaces,
-				$jmx_interfaces,
-				$ipmi_interfaces
-			]))->addClass(ZBX_STYLE_HOST_INTERFACES),
-			$host_is_discovered
-				? null
-				: new CDiv(
-					(new CButtonLink(_('Add')))
-						->addClass('add-interface')
-						->setMenuPopup([
-							'type' => 'submenu',
-							'data' => [
-								'submenu' => getAddNewInterfaceSubmenu()
-							]
-						])
-						->setAttribute('aria-label', _('Add new interface'))
-				)
+				(new CDiv([
+					renderInterfaceHeaders(),
+					$agent_interfaces,
+					$snmp_interfaces,
+					$jmx_interfaces,
+					$ipmi_interfaces
+				]))->addClass(ZBX_STYLE_HOST_INTERFACES),
+				$host_is_discovered
+					? null
+					: new CDiv(
+						(new CButtonLink(_('Add')))
+							->addClass('add-interface')
+							->setMenuPopup([
+								'type' => 'submenu',
+								'data' => [
+									'submenu' => getAddNewInterfaceSubmenu()
+								]
+							])
+							->setAttribute('aria-label', _('Add new interface'))
+					)
+			]))
+				->setAttribute('data-field-type', 'set')
+				->setAttribute('data-field-name', 'interfaces'),
+			(new CInput('hidden', 'main_interface_'.INTERFACE_TYPE_AGENT, 0))
+				->setAttribute('data-prevent-validation-on-change', 1)
+				->setAttribute('data-field-type', 'hidden'),
+			(new CInput('hidden', 'main_interface_'.INTERFACE_TYPE_SNMP, 0))
+				->setAttribute('data-prevent-validation-on-change', 1)
+				->setAttribute('data-field-type', 'hidden'),
+			(new CInput('hidden', 'main_interface_'.INTERFACE_TYPE_IPMI, 0))
+				->setAttribute('data-prevent-validation-on-change', 1)
+				->setAttribute('data-field-type', 'hidden'),
+			(new CInput('hidden', 'main_interface_'.INTERFACE_TYPE_JMX, 0))
+				->setAttribute('data-prevent-validation-on-change', 1)
+				->setAttribute('data-field-type', 'hidden')
 		])
 	])
 	->addItem([
@@ -277,7 +294,7 @@ $host_tab
 				'object_name' => 'proxies',
 				'multiple' => false,
 				'data' => $data['ms_proxy'],
-				'disabled' => $host_is_discovered,
+				'readonly' => $host_is_discovered,
 				'popup' => [
 					'parameters' => [
 						'srctbl' => 'proxies',
@@ -297,7 +314,7 @@ $host_tab
 				'object_name' => 'proxy_groups',
 				'multiple' => false,
 				'data' => $data['ms_proxy_group'],
-				'disabled' => $host_is_discovered,
+				'readonly' => $host_is_discovered,
 				'popup' => [
 					'parameters' => [
 						'srctbl' => 'proxy_groups',
@@ -315,6 +332,7 @@ $host_tab
 		new CFormField(
 			(new CCheckBox('status', HOST_STATUS_MONITORED))
 				->setChecked($data['host']['status'] == HOST_STATUS_MONITORED)
+				->setUncheckedValue(HOST_STATUS_NOT_MONITORED)
 		)
 	]);
 
@@ -328,10 +346,7 @@ if ($data['host']['assigned_proxyid'] != 0) {
 		->getUrl();
 
 	$proxy_name = $data['user']['can_edit_proxies']
-		? (new CLink($data['host']['assigned_proxy_name'], $proxy_url))
-			->addClass('js-edit-proxy')
-			->setAttribute('data-proxyid', $data['host']['assigned_proxyid'])
-			->setAttribute('data-action', 'proxy.edit')
+		? new CLink($data['host']['assigned_proxy_name'], $proxy_url)
 		: new CSpan($data['host']['assigned_proxy_name']);
 	$proxy_name->addClass('js-proxy-assigned');
 }
@@ -346,11 +361,14 @@ $host_tab->addItem([
 	]))->addClass('js-field-proxy-group-proxy')
 ]);
 
-$host_tab->addItem(
+$host_tab->addItem([
 	(new CTemplateTag('host-interface-row-tmpl'))->addItem(
-		new CPartial('configuration.host.interface.row')
+		new CPartial('configuration.host.interface.row', ['is_snmp' => false])
+	),
+	(new CTemplateTag('host-interface-row-snmp-tmpl'))->addItem(
+		new CPartial('configuration.host.interface.row', ['is_snmp' => true])
 	)
-);
+]);
 
 $ipmi_tab = (new CFormGrid())
 	->addItem([
@@ -404,7 +422,8 @@ $tags_tab = new CPartial('configuration.tags.tab', [
 	'tags' => $data['host']['tags'],
 	'with_automatic' => true,
 	'tabs_id' => 'host-tabs',
-	'tags_tab_id' => 'host-tags-tab'
+	'tags_tab_id' => 'host-tags-tab',
+	'has_inline_validation' => true
 ]);
 
 // Macros tab.
@@ -413,11 +432,13 @@ $macros_tab = (new CFormList('macrosFormList'))
 		->addValue(_('Host macros'), 0)
 		->addValue(_('Inherited and host macros'), 1)
 		->setModern(true)
+		->removeAttribute('data-field-type')
 	)
 	->addRow(null,
 		new CPartial('hostmacros.list.html', [
 			'macros' => $data['host']['macros'],
-			'readonly' => false
+			'readonly' => false,
+			'has_inline_validation' => true
 		]), 'macros_container'
 	);
 
@@ -426,19 +447,24 @@ $macro_row_tmpl = (new CTemplateTag('macro-row-tmpl'))
 		(new CRow([
 			(new CCol([
 				(new CTextAreaFlexible('macros[#{rowNum}][macro]', '', ['add_post_js' => false]))
+					->setErrorContainer('macros_#{rowNum}_error_container')
 					->addClass('macro')
 					->setWidth(ZBX_TEXTAREA_MACRO_WIDTH)
 					->setAttribute('placeholder', '{$MACRO}')
-					->disableSpellcheck(),
+					->disableSpellcheck()
+					->setErrorLabel(_('Macro')),
 				new CInput('hidden', 'macros[#{rowNum}][discovery_state]',
 					CControllerHostMacrosList::DISCOVERY_STATE_MANUAL
 				)
 			]))->addClass(ZBX_STYLE_TEXTAREA_FLEXIBLE_PARENT),
 			(new CCol(
-				new CMacroValue(ZBX_MACRO_TYPE_TEXT, 'macros[#{rowNum}]', '', false)
+				(new CMacroValue(ZBX_MACRO_TYPE_TEXT, 'macros[#{rowNum}]', '', false))
+					->setErrorContainer('macros_#{rowNum}_error_container')
+					->setErrorLabel(_('Value'))
 			))->addClass(ZBX_STYLE_TEXTAREA_FLEXIBLE_PARENT),
 			(new CCol(
 				(new CTextAreaFlexible('macros[#{rowNum}][description]', '', ['add_post_js' => false]))
+					->setErrorContainer('macros_#{rowNum}_error_container')
 					->setMaxlength(DB::getFieldLength('globalmacro', 'description'))
 					->setWidth(ZBX_TEXTAREA_MACRO_VALUE_WIDTH)
 					->setAttribute('placeholder', _('description'))
@@ -449,6 +475,15 @@ $macro_row_tmpl = (new CTemplateTag('macro-row-tmpl'))
 				])
 			))->addClass(ZBX_STYLE_NOWRAP)
 		]))->addClass('form_row')
+	)
+	->addItem(
+		new CRow(
+			(new CCol())
+				->setId('macros_#{rowNum}_error_container')
+				->addClass(ZBX_STYLE_ERROR_CONTAINER)
+				->setColSpan(4)
+		)
+
 	);
 
 $macro_row_inherited_tmpl = (new CTemplateTag('macro-row-tmpl-inherited'))
@@ -456,42 +491,46 @@ $macro_row_inherited_tmpl = (new CTemplateTag('macro-row-tmpl-inherited'))
 		(new CRow([
 			(new CCol([
 				(new CTextAreaFlexible('macros[#{rowNum}][macro]', '', ['add_post_js' => false]))
+					->setErrorContainer('macros_#{rowNum}_error_container')
 					->addClass('macro')
 					->setWidth(ZBX_TEXTAREA_MACRO_WIDTH)
 					->setAttribute('placeholder', '{$MACRO}')
-					->disableSpellcheck(),
+					->disableSpellcheck()
+					->setErrorLabel(_('Macro')),
 				new CInput('hidden', 'macros[#{rowNum}][inherited_type]', ZBX_PROPERTY_OWN),
 				new CInput('hidden', 'macros[#{rowNum}][discovery_state]',
 					CControllerHostMacrosList::DISCOVERY_STATE_MANUAL
 				)
 			]))->addClass(ZBX_STYLE_TEXTAREA_FLEXIBLE_PARENT),
 			(new CCol(
-				new CMacroValue(ZBX_MACRO_TYPE_TEXT, 'macros[#{rowNum}]', '', false)
+				(new CMacroValue(ZBX_MACRO_TYPE_TEXT, 'macros[#{rowNum}]', '', false))
+					->setErrorContainer('macros_#{rowNum}_error_container')
+					->setErrorLabel(_('Value'))
 			))->addClass(ZBX_STYLE_TEXTAREA_FLEXIBLE_PARENT),
 			(new CCol(
 				(new CButton('macros[#{rowNum}][remove]', _('Remove')))
 					->addClass(ZBX_STYLE_BTN_LINK)
 					->addClass('element-table-remove')
 			))->addClass(ZBX_STYLE_NOWRAP),
-			[
-				new CCol(
-					(new CDiv())
-						->addClass(ZBX_STYLE_OVERFLOW_ELLIPSIS)
-						->setAdaptiveWidth(ZBX_TEXTAREA_MACRO_VALUE_WIDTH)
-				),
-				new CCol(),
-				new CCol(
-					(new CDiv())
-						->addClass(ZBX_STYLE_OVERFLOW_ELLIPSIS)
-						->setAdaptiveWidth(ZBX_TEXTAREA_MACRO_VALUE_WIDTH)
-				)
-			]
+			new CCol(),
+			new CCol(
+				(new CDiv())
+					->addClass(ZBX_STYLE_OVERFLOW_ELLIPSIS)
+					->setAdaptiveWidth(ZBX_TEXTAREA_MACRO_VALUE_WIDTH)
+			),
+			new CCol(),
+			new CCol(
+				(new CDiv())
+					->addClass(ZBX_STYLE_OVERFLOW_ELLIPSIS)
+					->setAdaptiveWidth(ZBX_TEXTAREA_MACRO_VALUE_WIDTH)
+			)
 		]))->addClass('form_row')
 	)
 	->addItem(
 		(new CRow([
 			(new CCol(
 				(new CTextAreaFlexible('macros[#{rowNum}][description]', '', ['add_post_js' => false]))
+					->setErrorContainer('macros_#{rowNum}_error_container')
 					->setMaxlength(DB::getFieldLength('globalmacro', 'description'))
 					->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 					->setAttribute('placeholder', _('description'))
@@ -499,6 +538,14 @@ $macro_row_inherited_tmpl = (new CTemplateTag('macro-row-tmpl-inherited'))
 				->addClass(ZBX_STYLE_TEXTAREA_FLEXIBLE_PARENT)
 				->setColSpan(7)
 		]))->addClass('form_row')
+	)
+	->addItem(
+		new CRow(
+			(new CCol())
+				->setId('macros_#{rowNum}_error_container')
+				->addClass(ZBX_STYLE_ERROR_CONTAINER)
+				->setColSpan(4)
+		)
 	);
 
 $macros_tab
@@ -516,7 +563,7 @@ $inventory_tab = (new CFormGrid())
 				->addValue(_('Automatic'), HOST_INVENTORY_AUTOMATIC)
 				->setReadonly($host_is_discovered)
 				->setModern(),
-			$host_is_discovered ? new CInput('hidden', 'inventory_mode', $data['host']['inventory_mode']) : null
+			$host_is_discovered ? new CVar('inventory_mode', $data['host']['inventory_mode']) : null
 		])
 	]);
 
@@ -553,11 +600,7 @@ foreach ($data['inventory_fields'] as $inventory_no => $inventory_field) {
 			->getUrl();
 
 		$link = (new CLink($item_name, $item_url))
-			->setTitle(_s('This field is automatically populated by item "%1$s".', $item_name))
-			->setAttribute('data-itemid', $data['inventory_items'][$inventory_no]['itemid'])
-			->setAttribute('data-context', 'host')
-			->setAttribute('data-action', 'item.edit')
-			->addClass('js-update-item');
+			->setTitle(_s('This field is automatically populated by item "%1$s".', $item_name));
 
 		$inventory_item = (new CSpan([' ', LARR(), ' ', $link]))->addClass('populating_item');
 		$input_field->addClass('linked_to_item');
@@ -588,34 +631,34 @@ $is_psk_set = ($data['host']['tls_connect'] == HOST_ENCRYPTION_PSK || $tls_accep
 $encryption_tab = (new CFormGrid())
 	->addItem([
 		new CLabel(_('Connections to host')),
-		new CFormField(
+		new CFormField([
 			(new CRadioButtonList('tls_connect', (int) $data['host']['tls_connect']))
 				->addValue(_('No encryption'), HOST_ENCRYPTION_NONE)
 				->addValue(_('PSK'), HOST_ENCRYPTION_PSK)
 				->addValue(_('Certificate'), HOST_ENCRYPTION_CERTIFICATE)
-				->setModern(true)
 				->setReadonly($host_is_discovered)
-		)
+				->setModern(),
+			$host_is_discovered ? new CVar('tls_connect', (int) $data['host']['tls_connect']) : null
+		])
 	])
 	->addItem([
 		new CLabel(_('Connections from host')),
 		new CFormField([
 			(new CList([
 				(new CCheckBox('tls_in_none'))
-					->setChecked(($tls_accept & HOST_ENCRYPTION_NONE))
+					->setChecked($tls_accept & HOST_ENCRYPTION_NONE)
 					->setLabel(_('No encryption'))
 					->setReadonly($host_is_discovered),
 				(new CCheckBox('tls_in_psk'))
-					->setChecked(($tls_accept & HOST_ENCRYPTION_PSK))
+					->setChecked($tls_accept & HOST_ENCRYPTION_PSK)
 					->setLabel(_('PSK'))
 					->setReadonly($host_is_discovered),
 				(new CCheckBox('tls_in_cert'))
-					->setChecked(($tls_accept & HOST_ENCRYPTION_CERTIFICATE))
+					->setChecked($tls_accept & HOST_ENCRYPTION_CERTIFICATE)
 					->setLabel(_('Certificate'))
 					->setReadonly($host_is_discovered)
 			]))
-				->addClass(ZBX_STYLE_LIST_CHECK_RADIO),
-			new CInput('hidden', 'tls_accept', $tls_accept)
+				->addClass(ZBX_STYLE_LIST_CHECK_RADIO)
 		])
 	])
 	->addItem(
@@ -751,7 +794,7 @@ $output = [
 	'script_inline' => getPagePostJs().
 		$this->readJsFile('host.edit.js.php').
 		'host_edit_popup.init('.json_encode([
-			'form_name' => $data['form_name'],
+			'rules' => $data['js_validation_rules'],
 			'host_interfaces' => $data['host']['interfaces'],
 			'proxy_groupid' => $data['host']['proxy_groupid'],
 			'host_is_discovered' => ($data['host']['flags'] == ZBX_FLAG_DISCOVERY_CREATED),
