@@ -166,6 +166,12 @@ function itemGetValueTest(overlay) {
 	url.setArgument('action', 'popup.itemtest.getvalue');
 	url.setArgument(CSRF_TOKEN_NAME, <?= json_encode(CCsrfTokenHelper::get('itemtest')) ?>);
 
+	const macros = Object.fromEntries(
+		Object.values(form_data['macro_names'] || []).map(
+			(key, i) => [key, Object.values(form_data['macro_values'] || []).at(i)]
+		)
+	);
+
 	post_data = jQuery.extend(post_data, {
 		interface: {
 			address: interface ? interface['address'].trim() : '',
@@ -174,7 +180,7 @@ function itemGetValueTest(overlay) {
 			useip: interface ? interface['useip'] : null,
 			details: interface ? interface['details'] : null
 		},
-		macros: form_data['macros'],
+		macros: JSON.stringify(macros),
 		test_with: form_data['test_with'],
 		proxyid: form_data['proxyid'],
 		test_type: <?= $data['test_type'] ?>,
@@ -253,6 +259,14 @@ function itemCompleteTest(overlay) {
 	const interface = (form_data['interface'] !== undefined) ? form_data['interface'] : null;
 	const url = new Curl('zabbix.php');
 
+	const macros = {};
+
+	if (form_data.macro_names !== undefined) {
+		for (const [macro_index, macro_name] of Object.entries(form_data.macro_names)) {
+			macros[macro_name] = form_data.macro_values[macro_index];
+		}
+	}
+
 	url.setArgument('action', 'popup.itemtest.send');
 	url.setArgument(CSRF_TOKEN_NAME, <?= json_encode(CCsrfTokenHelper::get('itemtest')) ?>);
 
@@ -266,7 +280,7 @@ function itemCompleteTest(overlay) {
 			useip: interface ? interface['useip'] : null,
 			details: interface ? interface['details'] : null
 		},
-		macros: form_data['macros'],
+		macros: JSON.stringify(macros),
 		test_with: form_data['test_with'],
 		proxyid: form_data['proxyid'],
 		show_final_result: <?= $data['show_final_result'] ? 1 : 0 ?>,
@@ -527,11 +541,13 @@ function saveItemTestInputs() {
 		});
 	<?php endif ?>
 
-	jQuery('[name^=macros]').each(function(i, macro) {
-		var name = macro.name.toString();
-		macros[name.substr(7, name.length - 8)] = macro.value;
-	});
-	input_values.macros = macros;
+	if (form_data.macro_names !== undefined) {
+		for (const [macro_index, macro_name] of Object.entries(form_data.macro_names)) {
+			macros[macro_name] = form_data.macro_values[macro_index];
+		}
+	}
+
+	input_values.macros = JSON.stringify(macros);
 
 	<?php if ($data['step_obj'] == -2): ?>
 		$test_obj = jQuery('.overlay-dialogue-footer');
