@@ -1,6 +1,6 @@
 <?php
 /*
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -157,8 +157,9 @@ class CImage extends CApiService {
 		}
 
 		if (!$options['preservekeys']) {
-			$result = zbx_cleanHashes($result);
+			$result = array_values($result);
 		}
+
 		return $result;
 	}
 
@@ -242,7 +243,7 @@ class CImage extends CApiService {
 	 *
 	 * @throws APIException if the input is invalid
 	 */
-	private static function validateUpdate(array &$images, array &$db_images = null): void {
+	private static function validateUpdate(array &$images, ?array &$db_images = null): void {
 		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['imageid'], ['name']], 'fields' => [
 			'imageid' =>	['type' => API_ID, 'flags' => API_REQUIRED],
 			'name' =>		['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('images', 'name')],
@@ -295,7 +296,7 @@ class CImage extends CApiService {
 	 *
 	 * @throws APIException if the input is invalid
 	 */
-	private static function validateDelete(array &$imageids, array &$db_images = null): void {
+	private static function validateDelete(array &$imageids, ?array &$db_images = null): void {
 		$api_input_rules = ['type' => API_IDS, 'flags' => API_NOT_EMPTY, 'uniq' => true];
 
 		if (!CApiInputValidator::validate($api_input_rules, $imageids, '/', $error)) {
@@ -370,7 +371,7 @@ class CImage extends CApiService {
 	 *
 	 * @throws APIException if image names are not unique.
 	 */
-	private static function checkDuplicates(array $images, array $db_images = null): void {
+	private static function checkDuplicates(array $images, ?array $db_images = null): void {
 		$names = [];
 
 		foreach ($images as $image) {
