@@ -65,6 +65,9 @@ class CImportValidatorFactory extends CRegistryFactory {
 			},
 			'7.2' => function() use ($format): CXmlValidatorGeneral {
 				return new C72XmlValidator($format);
+			},
+			'7.4' => function() use ($format): CXmlValidatorGeneral {
+				return new C74XmlValidator($format);
 			}
 		]);
 	}
