@@ -1,6 +1,6 @@
 <?php declare(strict_types = 0);
 /*
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -18,55 +18,51 @@ use Widgets\SvgGraph\Includes\CWidgetFieldDataSet;
 
 ?>
 
-window.widget_svggraph_form = new class {
+window.widget_form = new class extends CWidgetForm {
 
 	/**
 	 * @type {Map<HTMLLIElement, CSortable>}
 	 */
 	#single_items_sortable = new Map();
 
+	/**
+	 * @type {number}
+	 */
+	#dataset_row_unique_id = 0;
+
 	init({form_tabs_id, color_palette, templateid}) {
 		colorPalette.setThemeColors(color_palette);
 
 		this._$overlay_body = jQuery('.overlay-dialogue-body');
-		this._form = document.getElementById('widget-dialogue-form');
+		this._form = this.getForm();
 		this._templateid = templateid;
 		this._dataset_wrapper = document.getElementById('data_sets');
 		this._any_ds_aggregation_function_enabled = false;
 
-		this._$overlay_body.on('scroll', () => {
-			const $preview = jQuery('.<?= ZBX_STYLE_SVG_GRAPH_PREVIEW ?>', this._$overlay_body);
-
-			if (!$preview.length) {
-				this._$overlay_body.off('scroll');
-				return;
-			}
-
-			if ($preview.offset().top < this._$overlay_body.offset().top && this._$overlay_body.height() > 400) {
-				jQuery('#svg-graph-preview').css('top',
-					this._$overlay_body.offset().top - $preview.offset().top
-				);
-				jQuery('.graph-widget-config-tabs .ui-tabs-nav').css('top', $preview.height());
-			}
-			else {
-				jQuery('#svg-graph-preview').css('top', 0);
-				jQuery('.graph-widget-config-tabs .ui-tabs-nav').css('top', 0);
-			}
-		});
+		this.#dataset_row_unique_id =
+			this._dataset_wrapper.querySelectorAll('.<?= ZBX_STYLE_LIST_ACCORDION_ITEM ?>').length - 1;
 
 		jQuery(`#${form_tabs_id}`)
-			.on('tabsactivate', () => jQuery.colorpicker('hide'))
-			.on('change', 'input, z-select, .multiselect', () => this.onGraphConfigChange());
+			.on('change', 'input, z-color-picker, z-select, .multiselect', () => this.onGraphConfigChange());
+
+		this._dataset_wrapper.addEventListener('input', e => {
+			if (e.target.matches('input[name$="[data_set_label]"]') || e.target.matches('input[name$="[timeshift]"]')) {
+				this.registerUpdateEvent();
+			}
+		});
 
 		this._datasetTabInit();
 		this._problemsTabInit();
 
-		this.onGraphConfigChange();
+		this._updateForm();
+
+		this.ready();
 	}
 
 	onGraphConfigChange() {
+		this.registerUpdateEvent();
+
 		this._updateForm();
-		this._updatePreview();
 	}
 
 	updateVariableOrder(obj, row_selector, var_prefix) {
@@ -87,45 +83,16 @@ window.widget_svggraph_form = new class {
 					}
 				});
 
-				jQuery(`[name^="${var_prefix}["]`, this)
-					.filter(function () {
-						return jQuery(this).attr('name').match(/[a-z]+\[\d+]\[[a-z_]+]/);
-					})
-					.each(function () {
-						jQuery(this).attr('name',
-							jQuery(this).attr('name').replace(/([a-z]+\[)\d+(]\[[a-z_]+])/, `$1${k + i}$2`)
-						);
-					});
-
-				jQuery(`[id^="${var_prefix}_"]`, this)
-					.filter(function () {
-						return jQuery(this).attr('id').match(/[a-z]+_\d+_[a-z_]+/);
-					})
-					.each(function () {
-						jQuery(this).attr('id',
-							jQuery(this).attr('id').replace(/([a-z]+_)\d+(_[a-z_]+)/, `$1${k + i}$2`)
-						);
-					});
-
-				jQuery(`label[for^="${var_prefix}_"]`, this)
-					.filter(function () {
-						return jQuery(this).attr('for').match(/[a-z]+_\d+_[a-z_]+/);
-					})
-					.each(function () {
-						jQuery(this).attr('for',
-							jQuery(this).attr('for').replace(/([a-z]+_)\d+(_[a-z_]+)/, `$1${k + i}$2`)
-						);
-					});
-
-				jQuery(`[id^="lbl_${var_prefix}_"]`, this)
-					.filter(function () {
-						return jQuery(this).attr('id').match(/lbl_[a-z]+_\d+_[a-z_]+/);
-					})
-					.each(function () {
-						jQuery(this).attr('id',
-							jQuery(this).attr('id').replace(/(lbl_[a-z]+_)\d+(_[a-z_]+)/, `$1${k + i}$2`)
-						);
-					});
+				['name', 'color-field-name', 'palette-field-name'].forEach(attr => {
+					jQuery(`[${attr}^="${var_prefix}["]`, this)
+						.filter(function () {
+							return jQuery(this).attr(attr).match(/[a-z]+\[\d+]\[[a-z_]+]/);
+						})
+						.each(function () {
+							const $this = jQuery(this);
+							$this.attr(attr, $this.attr(attr).replace(/([a-z]+\[)\d+(]\[[a-z_]+])/, `$1${k + i}$2`));
+						});
+				});
 			});
 		}
 	}
@@ -134,22 +101,32 @@ window.widget_svggraph_form = new class {
 		this._updateDatasetsLabel();
 
 		// Initialize vertical accordion.
-		jQuery(this._dataset_wrapper)
-			.on('focus', '.<?= CMultiSelect::ZBX_STYLE_CLASS ?> input.input', function() {
-				jQuery('#data_sets').zbx_vertical_accordion('expandNth',
-					jQuery(this).closest('.<?= ZBX_STYLE_LIST_ACCORDION_ITEM ?>').index()
+
+		const $data_sets = jQuery(this._dataset_wrapper);
+
+		$data_sets
+			.on('focus', '.<?= CMultiSelect::ZBX_STYLE_CLASS ?> input.input', function(e) {
+				const list_item = e.target.closest('.<?= ZBX_STYLE_LIST_ACCORDION_ITEM ?>');
+
+				if (list_item.classList.contains('<?= ZBX_STYLE_LIST_ACCORDION_ITEM_OPENED ?>')) {
+					return;
+				}
+
+				$data_sets.zbx_vertical_accordion('expandNth',
+					[...list_item.parentElement.children].indexOf(list_item)
 				);
 			})
-			.on('click', function(e) {
-				if (!e.target.classList.contains('color-picker-preview')) {
-					jQuery.colorpicker('hide');
+			.on('click', '.<?= ZBX_STYLE_LIST_ACCORDION_ITEM ?>', function(e) {
+				const list_item = e.target.closest('.<?= ZBX_STYLE_LIST_ACCORDION_ITEM ?>');
+
+				if (list_item.classList.contains('<?= ZBX_STYLE_LIST_ACCORDION_ITEM_OPENED ?>')) {
+					return;
 				}
 
 				if (e.target.classList.contains('js-click-expand')
-						|| e.target.classList.contains('color-picker-preview')
-						|| e.target.classList.contains('<?= ZBX_STYLE_BTN_GREY ?>')) {
-					jQuery('#data_sets').zbx_vertical_accordion('expandNth',
-						jQuery(e.target).closest('.<?= ZBX_STYLE_LIST_ACCORDION_ITEM ?>').index()
+						|| e.target.closest(`.${ZBX_STYLE_COLOR_PICKER}`) !== null) {
+					$data_sets.zbx_vertical_accordion('expandNth',
+						[...list_item.parentElement.children].indexOf(list_item)
 					);
 				}
 			})
@@ -177,7 +154,7 @@ window.widget_svggraph_form = new class {
 						message_block.style.display = 'none';
 					}
 
-					widget_svggraph_form._initSingleItemSortable(dataset);
+					widget_form._initSingleItemSortable(dataset);
 				}
 			})
 			.zbx_vertical_accordion({handler: '.<?= ZBX_STYLE_LIST_ACCORDION_ITEM_TOGGLE ?>'});
@@ -185,20 +162,6 @@ window.widget_svggraph_form = new class {
 		// Initialize rangeControl UI elements.
 		jQuery('.<?= CRangeControl::ZBX_STYLE_CLASS ?>', jQuery(this._dataset_wrapper)).rangeControl();
 
-		// Initialize pattern fields.
-		jQuery('.multiselect', jQuery(this._dataset_wrapper)).multiSelect();
-
-		for (const colorpicker of jQuery('.<?= ZBX_STYLE_COLOR_PICKER ?> input')) {
-			jQuery(colorpicker).colorpicker({
-				onUpdate: function(color) {
-					jQuery('.<?= ZBX_STYLE_COLOR_PREVIEW_BOX ?>',
-						jQuery(this).closest('.<?= ZBX_STYLE_LIST_ACCORDION_ITEM ?>')
-					).css('background-color', `#${color}`);
-				},
-				appendTo: '.overlay-dialogue-body'
-			});
-		}
-
 		this._dataset_wrapper.addEventListener('click', (e) => {
 			if (e.target.classList.contains('js-add-item')) {
 				this._selectItems();
@@ -244,8 +207,6 @@ window.widget_svggraph_form = new class {
 			}
 
 			this._initSingleItemSortable(this._getOpenedDataset());
-
-			this._updatePreview();
 		}
 
 		this._updateSingleItemsReferences();
@@ -335,29 +296,26 @@ window.widget_svggraph_form = new class {
 
 		const used_colors = [];
 
-		for (const color of this._form.querySelectorAll('.<?= ZBX_STYLE_COLOR_PICKER ?> input')) {
-			if (color.value !== '') {
-				used_colors.push(color.value);
+		for (const color_picker of this._form.querySelectorAll(`.${ZBX_STYLE_COLOR_PICKER}`)) {
+			if (color_picker.color !== '') {
+				used_colors.push(color_picker.color);
 			}
 		}
 
-		this._dataset_wrapper.insertAdjacentHTML('beforeend', template.evaluate({
-			rowNum: this._dataset_wrapper.querySelectorAll('.<?= ZBX_STYLE_LIST_ACCORDION_ITEM ?>').length,
+		const fragment = document.createRange().createContextualFragment(template.evaluate({
+			rowNum: ++this.#dataset_row_unique_id,
 			color: type == <?= CWidgetFieldDataSet::DATASET_TYPE_SINGLE_ITEM ?>
 				? ''
 				: colorPalette.getNextColor(used_colors)
 		}));
 
+		this._dataset_wrapper.append(fragment);
+
+		this.updateVariableOrder(this._dataset_wrapper, '.<?= ZBX_STYLE_LIST_ACCORDION_ITEM ?>', 'ds');
+		this._updateDatasetsLabel();
+
 		const dataset = this._getOpenedDataset();
 
-		for (const colorpicker of dataset.querySelectorAll('.<?= ZBX_STYLE_COLOR_PICKER ?> input')) {
-			jQuery(colorpicker).colorpicker({appendTo: '.overlay-dialogue-body'});
-		}
-
-		for (const multiselect of dataset.querySelectorAll('.multiselect')) {
-			jQuery(multiselect).multiSelect();
-		}
-
 		for (const range_control of dataset.querySelectorAll('.<?= CRangeControl::ZBX_STYLE_CLASS ?>')) {
 			jQuery(range_control).rangeControl();
 		}
@@ -368,6 +326,8 @@ window.widget_svggraph_form = new class {
 
 		this._initDataSetSortable();
 		this._updateForm();
+
+		this.registerUpdateEvent();
 	}
 
 	_cloneDataset() {
@@ -423,7 +383,6 @@ window.widget_svggraph_form = new class {
 		}
 
 		this._updateDatasetLabel(cloned_dataset);
-		this._updatePreview();
 	}
 
 	_removeDataSet(obj) {
@@ -463,7 +422,8 @@ window.widget_svggraph_form = new class {
 			this._sortable_data_set.on(CSortable.EVENT_SORT, () => {
 				this.updateVariableOrder(this._dataset_wrapper, '.<?= ZBX_STYLE_LIST_ACCORDION_ITEM ?>', 'ds');
 				this._updateDatasetsLabel();
-				this._updatePreview();
+
+				this.registerUpdateEvent();
 			});
 		}
 	}
@@ -522,7 +482,7 @@ window.widget_svggraph_form = new class {
 					srctbl: 'items',
 					srcfld1: 'itemid',
 					srcfld2: 'name',
-					dstfrm: widget_svggraph_form._form.id,
+					dstfrm: this._form.id,
 					dstfld1: `items_${dataset_index}_${row_index}_itemid`,
 					dstfld2: `items_${dataset_index}_${row_index}_name`,
 					numeric: 1,
@@ -538,7 +498,7 @@ window.widget_svggraph_form = new class {
 					srctbl: 'items',
 					srcfld1: 'itemid',
 					srcfld2: 'name',
-					dstfrm: widget_svggraph_form._form.id,
+					dstfrm: this._form.id,
 					dstfld1: `items_${dataset_index}_${row_index}_itemid`,
 					dstfld2: `items_${dataset_index}_${row_index}_name`,
 					numeric: 1,
@@ -566,7 +526,7 @@ window.widget_svggraph_form = new class {
 	_selectWidget(row = null, exclude_typed_references = []) {
 		const widgets = ZABBIX.Dashboard.getReferableWidgets({
 			type: CWidgetsData.DATA_TYPE_ITEM_ID,
-			widget_context: ZABBIX.Dashboard.getEditingWidgetContext()
+			widget_context: ZABBIX.Dashboard.getWidgetEditingContext()
 		});
 
 		widgets.sort((a, b) => a.getHeaderName().localeCompare(b.getHeaderName()));
@@ -651,15 +611,15 @@ window.widget_svggraph_form = new class {
 
 		const used_colors = [];
 
-		for (const color of this._form.querySelectorAll('.<?= ZBX_STYLE_COLOR_PICKER ?> input')) {
-			if (color.value !== '') {
-				used_colors.push(color.value);
+		for (const color_picker of this._form.querySelectorAll(`.${ZBX_STYLE_COLOR_PICKER}`)) {
+			if (color_picker.color !== '') {
+				used_colors.push(color_picker.color);
 			}
 		}
 
-		jQuery(`#items_${dataset_index}_${items_new_index}_color`)
-			.val(colorPalette.getNextColor(used_colors))
-			.colorpicker();
+		row.querySelector(`.${ZBX_STYLE_COLOR_PICKER}`).color = colorPalette.getNextColor(used_colors);
+
+		this.registerUpdateEvent();
 	}
 
 	_removeSingleItem(element) {
@@ -669,7 +629,8 @@ window.widget_svggraph_form = new class {
 
 		this._updateSingleItemsOrder(dataset);
 		this._initSingleItemSortable(dataset);
-		this._updatePreview();
+
+		this.registerUpdateEvent();
 	}
 
 	_initSingleItemSortable(dataset) {
@@ -689,7 +650,8 @@ window.widget_svggraph_form = new class {
 
 		sortable.on(CSortable.EVENT_SORT, () => {
 			this._updateSingleItemsOrder(dataset);
-			this._updatePreview();
+
+			this.registerUpdateEvent();
 		});
 
 		this.#single_items_sortable.set(dataset, sortable);
@@ -699,7 +661,7 @@ window.widget_svggraph_form = new class {
 		const widgets = ZABBIX.Dashboard
 			.getReferableWidgets({
 				type: CWidgetsData.DATA_TYPE_ITEM_ID,
-				widget_context: ZABBIX.Dashboard.getEditingWidgetContext()
+				widget_context: ZABBIX.Dashboard.getWidgetEditingContext()
 			})
 			.reduce((map, widget) => map.set(widget.getFields().reference, widget.getHeaderName()), new Map());
 
@@ -732,8 +694,6 @@ window.widget_svggraph_form = new class {
 	}
 
 	_updateSingleItemsOrder(dataset) {
-		jQuery.colorpicker('destroy', jQuery('.single-item-table .<?= ZBX_STYLE_COLOR_PICKER ?> input', dataset));
-
 		const dataset_index = dataset.getAttribute('data-set');
 
 		for (const row of dataset.querySelectorAll('.single-item-table-row')) {
@@ -743,11 +703,6 @@ window.widget_svggraph_form = new class {
 			row.querySelector('.table-col-name a').id = `${prefix}_name`;
 			row.querySelector('.table-col-action input[name$="[itemids][]"]').id = `${prefix}_itemid`;
 			row.querySelector('.table-col-action input[name$="[references][]"]').id = `${prefix}_reference`;
-
-			const colorpicker = row.querySelector('.single-item-table .<?= ZBX_STYLE_COLOR_PICKER ?> input');
-
-			colorpicker.id = `${prefix}_color`;
-			jQuery(colorpicker).colorpicker({appendTo: '.overlay-dialogue-body'});
 		}
 	}
 
@@ -890,7 +845,7 @@ window.widget_svggraph_form = new class {
 			lefty_checkbox.checked = true;
 		}
 
-		for (const element of document.querySelectorAll('#lefty_min, #lefty_max, #lefty_units')) {
+		for (const element of document.querySelectorAll('#lefty_scale, #lefty_min, #lefty_max, #lefty_units')) {
 			element.disabled = !lefty_on;
 		}
 
@@ -906,7 +861,7 @@ window.widget_svggraph_form = new class {
 			righty_checkbox.checked = true;
 		}
 
-		for (const element of document.querySelectorAll('#righty_min, #righty_max, #righty_units')) {
+		for (const element of document.querySelectorAll('#righty_scale, #righty_min, #righty_max, #righty_units')) {
 			element.disabled = !righty_on;
 		}
 
@@ -931,216 +886,10 @@ window.widget_svggraph_form = new class {
 
 		document.querySelector('[for=legend_lines]')
 			.textContent = document.querySelector('[name=legend_lines_mode]:checked').value === '1'
-				? '<?= _('Maximum number of rows') ?>'
-				: '<?= _('Number of rows') ?>';
+				? <?= json_encode(_('Maximum number of rows')) ?>
+				: <?= json_encode(_('Number of rows')) ?>;
 
 		// Trigger event to update tab indicators.
 		document.getElementById('tabs').dispatchEvent(new Event(TAB_INDICATOR_UPDATE_EVENT));
 	}
-
-	#update_preview_abort_controller = null;
-	#update_preview_loading_timeout = null;
-
-	_updatePreview() {
-		if (this.#update_preview_abort_controller !== null) {
-			this.#update_preview_abort_controller.abort();
-		}
-
-		if (this.#update_preview_loading_timeout !== null) {
-			clearTimeout(this.#update_preview_loading_timeout);
-		}
-
-		const preview = document.getElementById('svg-graph-preview');
-		const preview_container = preview.parentElement;
-		const preview_computed_style = getComputedStyle(preview);
-		const contents_width = Math.floor(parseFloat(preview_computed_style.width));
-		const contents_height = Math.floor(parseFloat(preview_computed_style.height)) - 10;
-
-		const fields = getFormFields(this._form);
-
-		fields.override_hostid = this.#resolveOverrideHostId();
-		fields.time_period = this.#resolveTimePeriod(fields.time_period);
-
-		if (fields.ds !== undefined) {
-			for (const [dataset_key, dataset] of Object.entries(fields.ds)) {
-				if (dataset.dataset_type != '<?= CWidgetFieldDataSet::DATASET_TYPE_SINGLE_ITEM ?>') {
-					continue;
-				}
-
-				const dataset_new = {
-					...dataset,
-					itemids: [],
-					color: []
-				};
-
-				if (dataset.itemids !== undefined) {
-					for (const [item_index, itemid] of dataset.itemids.entries()) {
-						if (itemid === '0') {
-							const resolved_itemid = this.#resolveWidget(dataset.references[item_index]);
-
-							if (resolved_itemid !== null) {
-								dataset_new.itemids.push(resolved_itemid);
-								dataset_new.color.push(dataset.color[item_index]);
-							}
-						}
-						else {
-							dataset_new.itemids.push(itemid);
-							dataset_new.color.push(dataset.color[item_index]);
-						}
-					}
-				}
-
-				delete dataset_new.references;
-
-				fields.ds[dataset_key] = dataset_new;
-			}
-		}
-
-		const data = {
-			templateid: this._templateid ?? undefined,
-			fields,
-			preview: 1,
-			contents_width,
-			contents_height
-		};
-
-		const curl = new Curl('zabbix.php');
-
-		curl.setArgument('action', 'widget.svggraph.view');
-
-		this.#update_preview_loading_timeout = setTimeout(() => {
-			this.#update_preview_loading_timeout = null;
-			preview_container.classList.add('is-loading');
-		}, 1000);
-
-		const abort_controller = new AbortController();
-
-		this.#update_preview_abort_controller = abort_controller;
-
-		fetch(curl.getUrl(), {
-			method: 'POST',
-			headers: {'Content-Type': 'application/json'},
-			body: JSON.stringify(data),
-			signal: abort_controller.signal
-		})
-			.then((response) => response.json())
-			.then((response) => {
-				if ('error' in response) {
-					throw {error: response.error};
-				}
-
-				for (const element of this._form.parentNode.children) {
-					if (element.matches('.msg-good, .msg-bad, .msg-warning')) {
-						element.parentNode.removeChild(element);
-					}
-				}
-
-				if (this.#update_preview_loading_timeout !== null) {
-					clearTimeout(this.#update_preview_loading_timeout);
-					this.#update_preview_loading_timeout = null;
-				}
-
-				preview_container.classList.remove('is-loading');
-
-				if ('body' in response) {
-					preview.innerHTML = response.body;
-					preview.setAttribute('unselectable', 'on');
-					preview.style.userSelect = 'none';
-				}
-			})
-			.catch((exception) => {
-				if (abort_controller.signal.aborted) {
-					return;
-				}
-
-				for (const element of this._form.parentNode.children) {
-					if (element.matches('.msg-good, .msg-bad, .msg-warning')) {
-						element.parentNode.removeChild(element);
-					}
-				}
-
-				if (this.#update_preview_loading_timeout !== null) {
-					clearTimeout(this.#update_preview_loading_timeout);
-					this.#update_preview_loading_timeout = null;
-				}
-
-				preview_container.classList.remove('is-loading');
-
-				let title;
-				let messages = [];
-
-				if (typeof exception === 'object' && 'error' in exception) {
-					title = exception.error.title;
-					messages = exception.error.messages;
-				}
-				else {
-					title = <?= json_encode(_('Unexpected server error.')) ?>;
-				}
-
-				const message_box = makeMessageBox('bad', messages, title)[0];
-
-				this._form.parentNode.insertBefore(message_box, this._form);
-			});
-	}
-
-	#resolveWidget(typed_reference) {
-		const {reference, type} = CWidgetBase.parseTypedReference(typed_reference);
-
-		const data = ZABBIX.EventHub.getData({
-			context: 'dashboard',
-			event_type: 'broadcast',
-			reference,
-			type
-		});
-
-		if (data !== undefined && data.length === 1) {
-			return data[0];
-		}
-
-		return null;
-	}
-
-	#resolveTimePeriod(time_period_field) {
-		if ('from' in time_period_field && 'to' in time_period_field) {
-			return time_period_field;
-		}
-
-		let time_period;
-
-		if (CWidgetBase.FOREIGN_REFERENCE_KEY in time_period_field) {
-			const {reference} = CWidgetBase.parseTypedReference(
-				time_period_field[CWidgetBase.FOREIGN_REFERENCE_KEY]
-			);
-
-			time_period = ZABBIX.EventHub.getData({
-				context: 'dashboard',
-				event_type: 'broadcast',
-				reference,
-				type: CWidgetsData.DATA_TYPE_TIME_PERIOD
-			});
-		}
-
-		if (time_period === undefined || time_period === null) {
-			time_period = ZABBIX.EventHub.getData({
-				context: 'dashboard',
-				event_type: 'broadcast',
-				reference: CDashboard.REFERENCE_DASHBOARD,
-				type: CWidgetsData.DATA_TYPE_TIME_PERIOD
-			});
-		}
-
-		return {
-			from: time_period.from,
-			to: time_period.to
-		};
-	}
-
-	#resolveOverrideHostId() {
-		return ZABBIX.EventHub.getData({
-			context: 'dashboard',
-			event_type: 'broadcast',
-			reference: CDashboard.REFERENCE_DASHBOARD,
-			type: CWidgetsData.DATA_TYPE_HOST_ID
-		});
-	}
 };
